//*****************************************************************************
//    THE SOFTWARE INCLUDED IN THIS FILE IS FOR GUIDANCE ONLY.
//    AUTHOR SHALL NOT BE HELD LIABLE FOR ANY DIRECT, INDIRECT
//    OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
//    FROM USE OF THIS SOFTWARE.
//
//    PROGRAM ZAWARTY W TYM PLIKU PRZEZNACZONY JEST WYLACZNIE
//    DO CELOW SZKOLENIOWYCH. AUTOR NIE PONOSI ODPOWIEDZIALNOSCI
//    ZA ZADNE EWENTUALNE, BEZPOSREDNIE I POSREDNIE SZKODY
//    WYNIKLE Z JEGO WYKORZYSTANIA.
//******************************************************************************


#include "HMC5883L.h"
#include "stm32f10x_gpio.h"
#include "stm32f10x_i2c.h"

#define HMC5883L_Addr 0x3C        // 0011 101x

//write 3D read 1e


void HMC5883L_Configuration(void)
{ //GPIO and SPI configuration
  GPIO_InitTypeDef  GPIO_InitStructure; 
  I2C_InitTypeDef I2C_InitStructure;


  //GPIO Config
  GPIO_InitStructure.GPIO_Pin = HMC5883L_SCL | HMC5883L_SDA;      //I2C - SCL & SDA
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_OD;
  GPIO_Init(HMC5883L_I2C_GPIO, &GPIO_InitStructure);
  
  //I2C Config
  I2C_InitStructure.I2C_Mode = I2C_Mode_I2C;                //I2C mode
  I2C_InitStructure.I2C_DutyCycle = I2C_DutyCycle_2;        //High to low state ratio of SCK, important for >100kHz transmission speed
  I2C_InitStructure.I2C_OwnAddress1 = 0x00;                 //I2C address at MCU side (if it works as slave)
  I2C_InitStructure.I2C_Ack = I2C_Ack_Enable;               //Acknowledge enable
  I2C_InitStructure.I2C_AcknowledgedAddress = I2C_AcknowledgedAddress_7bit; //Addressing mode
  I2C_InitStructure.I2C_ClockSpeed=1000;                  //Clock frequency
  I2C_Init(HMC5883L_I2C,&I2C_InitStructure);

  I2C_Cmd(HMC5883L_I2C, ENABLE); 
}

char HMC5883L_Initialize(void)
{
	int error=0;
  unsigned char RegVal;
  error=SetScale(0.88); 	
	HMC5883L_WriteRegister(0x02, 0x00);   //Cykliczny pomiar
	HMC5883L_ReadRegister(0x01,&RegVal);
  if (RegVal!=0x00 || error == 1)
	{
		return 1;
	}
	return 0;
}


char SetScale(float gauss)
{
	extern float m_Scale;
	uint8_t regValue = 0x00;
	if(gauss == (float)0.88)
	{
		regValue = 0x00;
		m_Scale = 0.73;
	}
	else if(gauss == (float)1.3)
	{
		regValue = 0x01;
		m_Scale = 0.92;
	}
	else if(gauss == (float)1.9)
	{
		regValue = 0x02;
		m_Scale = 1.22;
	}
	else if(gauss == (float)2.5)
	{
		regValue = 0x03;
		m_Scale = 1.52;
	}
	else if(gauss == (float)4.0)
	{
		regValue = 0x04;
		m_Scale = 2.27;
	}
	else if(gauss == (float)4.7)
	{
		regValue = 0x05;
		m_Scale = 2.56;
	}
	else if(gauss == (float)5.6)
	{
		regValue = 0x06;
		m_Scale = 3.03;
	}
	else if(gauss == (float)8.1)
	{
		regValue = 0x07;
		m_Scale = 4.35;
	}
	else
		return 1;
	
	// Setting is in the top 3 bits of the register.
	regValue = regValue << 5;
	HMC5883L_WriteRegister(0x01, regValue); 
	return 0;
}

void HMC5883L_ReadRegister(char addr,unsigned char *v)
{
  //Reads one register value

  I2C_GenerateSTART(HMC5883L_I2C,ENABLE);
  //Test on EV5 and clear it
  while(!I2C_CheckEvent(HMC5883L_I2C, I2C_EVENT_MASTER_MODE_SELECT));  
  //Send HMC5883L address, set I2C master in transmiter mode
  I2C_Send7bitAddress(HMC5883L_I2C, HMC5883L_Addr, I2C_Direction_Transmitter);
  //Test on EV6 and clear it
  while(!I2C_CheckEvent(HMC5883L_I2C, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED));
  //Send HMC5883L local register address, which will be read
  I2C_SendData(HMC5883L_I2C, addr);
  // Test on EV8 and clear it
  while(!I2C_CheckEvent(HMC5883L_I2C, I2C_EVENT_MASTER_BYTE_TRANSMITTED));

  //Re-generate START, transmition from slave beginning
  I2C_GenerateSTART(HMC5883L_I2C,ENABLE);
  //Test on EV5 and clear it
  while(!I2C_CheckEvent(HMC5883L_I2C, I2C_EVENT_MASTER_MODE_SELECT)); 
  //Send HMC5883L address, set I2C master in receiver mode
  I2C_Send7bitAddress(HMC5883L_I2C, HMC5883L_Addr, I2C_Direction_Receiver);
  //Only one byte will be received
  //NAck must be configured just before checking EV6 -> disable Acknowledge
  I2C_AcknowledgeConfig(HMC5883L_I2C, DISABLE);
  //Test on EV6 and clear it
  while(!I2C_CheckEvent(HMC5883L_I2C, I2C_EVENT_MASTER_RECEIVER_MODE_SELECTED));
  //Generate STOP Condition
  I2C_GenerateSTOP(HMC5883L_I2C,ENABLE);
  //Test on EV7 and clear it - Wait until DataN is in Shift register
  while(!I2C_CheckEvent(HMC5883L_I2C, I2C_EVENT_MASTER_BYTE_RECEIVED));
  //Read DataN from DR register
  *v=I2C_ReceiveData(HMC5883L_I2C);
  //Enable Acknowledge for next transmission 
  I2C_AcknowledgeConfig(HMC5883L_I2C, ENABLE);
}



void HMC5883L_WriteRegister(char addr,char v)
{
  //Writes one register value 

  I2C_GenerateSTART(HMC5883L_I2C,ENABLE);
  //Test on EV5 and clear it
  while(!I2C_CheckEvent(HMC5883L_I2C, I2C_EVENT_MASTER_MODE_SELECT));  
  //Send HMC5883L address, set I2C master in transmiter mode
  I2C_Send7bitAddress(HMC5883L_I2C, HMC5883L_Addr, I2C_Direction_Transmitter);
  //Test on EV6 and clear it
  while(!I2C_CheckEvent(HMC5883L_I2C, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED));
  //Send HMC5883L local register address, which will be written 
  I2C_SendData(HMC5883L_I2C, addr);
  //Test on EV8 and clear it
  while(!I2C_CheckEvent(HMC5883L_I2C, I2C_EVENT_MASTER_BYTE_TRANSMITTED));
  //Send new register value
  I2C_SendData(HMC5883L_I2C, v);
  //Test on EV8 and clear it
  while(!I2C_CheckEvent(HMC5883L_I2C, I2C_EVENT_MASTER_BYTE_TRANSMITTED));
  //Generate STOP Condition
  I2C_GenerateSTOP(HMC5883L_I2C,ENABLE); 
}

void HMC5883L_GetPosition(signed short * x,signed short * y, signed short * z)
{
  //reads 5 registers values into buffer in multiple bytes mode, buffer[0]=x, [2]=z, [3]=y
  unsigned char Bufor[6]={0};
  unsigned int NumByteToReadN, i;


  //Check, if I2C is free
  while(I2C_GetFlagStatus(I2C1, I2C_FLAG_BUSY));

  I2C_GenerateSTART(I2C1,ENABLE);
  //Test on EV5 and clear it
  while(!I2C_CheckEvent(HMC5883L_I2C, I2C_EVENT_MASTER_MODE_SELECT));  
  //Send HMC5883L address, set I2C master in transmiter mode
  I2C_Send7bitAddress(HMC5883L_I2C, HMC5883L_Addr, I2C_Direction_Transmitter);
  //Test on EV6 and clear it
  while(!I2C_CheckEvent(HMC5883L_I2C, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED));
  //Send base register address, set address autoincrement
  I2C_SendData(HMC5883L_I2C, 0x03);
  //Test on EV8 and clear it
  while(!I2C_CheckEvent(HMC5883L_I2C, I2C_EVENT_MASTER_BYTE_TRANSMITTED));

  //Re-generate START, transmition from slave beginning
  I2C_GenerateSTART(HMC5883L_I2C,ENABLE);
  //Test on EV5 and clear it
  while(!I2C_CheckEvent(HMC5883L_I2C, I2C_EVENT_MASTER_MODE_SELECT)); 
  //Send HMC5883L address, set I2C master in receiver mode
  I2C_Send7bitAddress(HMC5883L_I2C, HMC5883L_Addr, I2C_Direction_Receiver);
  //Test on EV6 and clear it 
  while(!I2C_CheckEvent(HMC5883L_I2C, I2C_EVENT_MASTER_RECEIVER_MODE_SELECTED));
 
  NumByteToReadN=6;  //Read 5 consecutive registers
  i=0;               //Current read count
  while(NumByteToReadN) {
    //Before receiving last byte, disable acknowledge and generate stop
    if(NumByteToReadN == 1) {
      I2C_AcknowledgeConfig(HMC5883L_I2C, DISABLE);
      I2C_GenerateSTOP(HMC5883L_I2C, ENABLE);
    }
    //Test on EV7 and clear it
    while(!I2C_CheckEvent(HMC5883L_I2C, I2C_EVENT_MASTER_BYTE_RECEIVED));
    //Read a byte from the LI35 
    Bufor[i] = I2C_ReceiveData(HMC5883L_I2C);
    i++; 
    NumByteToReadN--;        
  }

  //Enable Acknowledge for next transmission 
  I2C_AcknowledgeConfig(HMC5883L_I2C, ENABLE);

  //Assign data to axis variables
	

  *x=(signed short)((Bufor[0]<<8)|Bufor[1]);
  *z=(signed short)((Bufor[2]<<8)|Bufor[3]);
  *y=(signed short)((Bufor[4]<<8)|Bufor[5]);

}
