//**********************************************//
//******************HMC5883L******************//
//**********************************************//

//W folderze do ktrego rozpakujemy archiwum ZL27ARM_Kompas.zip musi 
//znajdowac sie folder z biblioteka "STM32F10x_StdPeriph_Lib_V3.5.0"
//w celu poprawnej kompilacji programu.

//Do dzialania programu potrzebny jest modul KAmod TFT2
//polaczenia do wykonania pomiedzy modulem a zestawem ZL27ARM

//****************ZL27ARM - KAmodTFT2*****************//  
//JP6 PA4           <-> Con1 SCE
//JP6 PA5           <-> Con1 CLK
//JP6 PA7           <-> Con1 DIN
//JP6 PA8           <-> Con1 RES
//JP6 GND           <-> Con1 GND
//JP6 +3,3V (GPIOB) <-> Con1 +V


//**************HMC5883L - ZL27ARM*************//
//Con1_SCL - JP5(PB6)
//Con1_SDA - JP5(PB7)
//Con1_+V - JP5(+3.3V)
//Con1_GND - JP5(GND)

//****************ZL27ARM - ZL27ARM*****************//
//JP5(PB12) - JP7(SPK)


#include "stm32f10x.h"
#include "Inicjalizacja.h"
#include "HMC5883L.h"
#include "tft_pcf8833_lib.h"
#include "roza.h"
#include "font1.h"
#include "font2.h"
#include "Sprzet.h"

#include "stdio.h"
#include "stdlib.h"
#include "math.h"

void Opcja_Info(void);
void Opcja_Kalibracja(void);
void Opcja_Kompas(void);
void Opcja_Krzywe(void);

//Zmienne dla danych z czujnika
float m_Scale;
signed short offsetx,offsety;
volatile unsigned char losowo;

int main(void)
{
  volatile unsigned long int i;
	extern volatile unsigned char losowo;
	
  signed char pozycja=1;
  signed char aktualizacja=0;
  const unsigned char *menu[6] = {"Czytaj to!!!\0", "Kalibracja\0", "Kompas\0","Krzywe\0"};
	
	//Konfiguracja systemu
	RCC_Configuration();
  NVIC_Configuration();
	GPIO_Configuration();
	
	//Wylaczenie LED
	GPIO_ResetBits(LED_PORT, LED1 | LED2 | LED3 | LED4 | LED5 | LED6 | LED7 | LED8);	
	
	HMC5883L_Configuration();
	TFTN_HwConfig();
  TFTN_Init();
	TFTN_Clear(0x000);

  while (HMC5883L_Initialize())
  GPIO_WriteBit(LED_PORT, LED1, SET);       //magnetometr zainicjalizowany 
  


	while(1)
	{
		//Obsluga menu
		if (!GPIO_ReadInputDataBit(JOY_SW_PORT,JOY_UP) && (pozycja>1)){
      aktualizacja=pozycja;
      pozycja--;
    }
    if (!GPIO_ReadInputDataBit(JOY_SW_PORT, JOY_DOWN) && (pozycja<4)){
      aktualizacja=pozycja;
      pozycja++;
    }

    //ENTER joysticka
    if (!GPIO_ReadInputDataBit(JOY_PORT_ENTER, JOY_ENTER)){
      switch (pozycja){
        case 1: Opcja_Info(); break;
        case 2: Opcja_Kalibracja(); break;
        case 3: Opcja_Kompas(); break;
				case 4: Opcja_Krzywe(); break;
      }
      aktualizacja=0;
    }                          


		//Rysowanie menu
    if (aktualizacja>=0){
      if (aktualizacja==0) {
        TFTN_Clear(0x000);
        TFTN_FillRect(  0, 0, 131, 18, 0x555);
        TFTN_WriteXY("MENU\0",45,2,0xFFF,0x555, font2);
      }
      for (i=1;i<=4;i++){
        if ((i==aktualizacja)||(aktualizacja==0)) {        
          TFTN_FillRect(  0, i*18+8, 131, i*18+8+18, 0x000);
          TFTN_WriteXY(menu[i-1],5,i*18+10,0xFFF,0x000, font2);
        }
        if (i==pozycja) {
          TFTN_FillRect(  0, i*18+8, 131, i*18+8+18, 0x000);
          TFTN_WriteXY(menu[i-1],5,i*18+10,0xF00,0x000, font2);
        }
      }
      aktualizacja=-1;
    }

    for (i=0;i<100000ul;i++); 	
		losowo++;//Zmienna dla funkcji "Krzywe"
  };
	
}
		
void Opcja_Info(void){
  volatile unsigned long int i;

  TFTN_Clear(0x000);
  TFTN_WriteXY("     DEMO    \0"   ,  5, 10,0xFFF,0x000, font2);
  TFTN_WriteXY(" magnetometru\0",  5, 26,0xFFF,0x000, font2);
  TFTN_WriteXY("    HMC588L\0"   ,  5, 42,0xFFF,0x000, font2);
  TFTN_WriteXY("    UWAGA!!!\0"   ,  5, 68,0xFFF,0x000, font2);
  TFTN_WriteXY("  Dla wiarygodnego\0",  5, 82,0xFFF,0x000, font1);
  TFTN_WriteXY(" pomiaru ustaw os Z\0"   ,  5, 90,0xFFF,0x000, font1);
  TFTN_WriteXY("czujnika prostopadle\0",5,98,0xFFF,0x000, font1);
	TFTN_WriteXY("   do ziemi oraz\0",5,106,0xFFF,0x000, font1);
  TFTN_WriteXY("wykonaj kalibracje!!\0",5,114,0xFFF,0x000, font1);
	//Czekaj na JOY_DOWN
	while(GPIO_ReadInputDataBit(JOY_SW_PORT, JOY_DOWN)) ;
}


void Opcja_Kalibracja(void){
	volatile unsigned long int i;
	volatile unsigned long int petla =500; //Czas kalibracji
	signed short x, y,z;
	signed short xmin=32767 ,xmax=-32768,ymin=32767,ymax=-32768;
	extern signed short offsetx,offsety;
	
	TFTN_Clear(0x000);
  TFTN_WriteXY("  KALIBRACJA    \0"   ,  5, 10,0xFFF,0x000, font2);
  TFTN_WriteXY("   UWAGA!!!  "   ,  5, 35,0xFFF,0x000, font2);
  TFTN_WriteXY("W czasie trwania\0"   ,  5, 55,0xFFF,0x000, font1);
  TFTN_WriteXY("sygnalu dzwiekowego\0",  5, 63,0xFFF,0x000, font1);
  TFTN_WriteXY("dokonaj minimalnie \0"   ,  5, 71,0xFFF,0x000, font1);
  TFTN_WriteXY("jednego obrotu \0",5,79,0xFFF,0x000, font1);
	TFTN_WriteXY("czujnika wzgledem osi\0",5,87,0xFFF,0x000, font1);
	TFTN_WriteXY("'Z' ustawionej do \0",5,95,0xFFF,0x000, font1);
	TFTN_WriteXY("ziemi prostopadle\0",5,103,0xFFF,0x000, font1);
	TFTN_WriteXY("Wcisnij JOY_ENTER\0",5,121,0xFFF,0x000, font1);
	//Czekaj na JOY_ENTER
	while(GPIO_ReadInputDataBit(JOY_PORT_ENTER, JOY_ENTER));
	
	//Wyznaczanie minimalnych i maksymalnych wartosci pola magnetycznego na osiach X i Y
	//po wykonaniu co najminej jednego obrotu czujnika wokol osi Z ustawionej prostopadle 
	//do ziemi
	while (petla)
	{
		HMC5883L_GetPosition(&x, &y, &z);
		
		if (x<xmin) xmin=x;
		if (x>xmax) xmax=x;
		
		if (y<ymin) ymin=y;
		if (y>ymax) ymax=y; 
		
		petla--;
		
		//W czasie kalibracji Speaker wydaje dzwiek
		GPIO_WriteBit(GPIOB, GPIO_Pin_12, (BitAction)(1-GPIO_ReadOutputDataBit(GPIOB, GPIO_Pin_12)));
	}
	
	//Przesuniecie ukladu wspolrzednych tak aby
	//jego srodek znajdowal sie w polowie roznicy minimalnego i maksymalnego
	//pola magnetycznego odczytanego przez magnetometr dla osi X oraz Y
	offsetx=abs((xmin+xmax)*0.5);
	if (abs(xmin)<xmax) offsetx=-offsetx;
	
	offsety=abs((ymin+ymax)*0.5);
	if (abs(ymin)<ymax) offsety=-offsety;
	
}


void Opcja_Kompas(void){
	
  volatile unsigned long int i;
	signed short x, y, z;
	float scaledx, scaledy, heading;
	float declinationAngle = 0.0880;
	float headingDegrees=0;
  unsigned char Tekst[4] = {"\0"};

	
	extern signed short offsetx,offsety;
	
	volatile signed short k,srednia;
	volatile int l,xn=0,xs=0,yn=0,ys=0,xw=0,yw=0,xe=0,ye=0;
	volatile int xns,xss,yns,yss,xws,yws,xes,yes ;
	
	volatile int xo1,yo1,xo2,yo2,xo3,yo3,xo4,yo4;
	volatile int xo1s,yo1s,xo2s,yo2s,xo3s,yo3s,xo4s,yo4s;
	
	float stopien = (2 * 3.1415926) / 360; 

	//Rysowanie rozy wiatrow
	TFTN_Clear(0x000);
	TFTN_WriteBMP(roza, 0, 0, 132, 132);	
	
	//Kompas dziala do momentu wcisniecia JOY_DOWN
	while(GPIO_ReadInputDataBit(JOY_SW_PORT, JOY_DOWN))
	{
		k=(signed short)headingDegrees;
		
		//Stare wspolrzedne rownaja sie nowym
		//celem wyczyszczenia starych danych z wyswietlacza
		xns=xn;
		yns=yn;
		xss=xs;
		yss=ys;
		xws=xw;
		yws=yw;
		xes=xe;
		yes=ye;
	
		xo1s=xo1;
		yo1s=yo1;
		xo2s=xo2;
		yo2s=yo2;
		xo3s=xo3;
		yo3s=yo3;
		xo4s=xo4;
	  yo4s=yo4;
	
		//Obliczanie nowych wspolrzednych dla rysowania
		//N,S,W,E,o,o,o,o w odniesieniu do kierunku geograficznego
		xn = (int)(62 + 59 * sin(stopien*k));
		yn = (int)(60 + 59 * cos(stopien*k));
	
		xo1 = (int)(62 + 59 * sin(stopien*(k+45)));
		yo1 = (int)(60 + 59 * cos(stopien*(k+45)));
	
		xo2 = (int)(62 + 59 * sin(stopien*(k+135)));
		yo2 = (int)(60 + 59 * cos(stopien*(k+135)));
	
		xo3 = (int)(62 + 59 * sin(stopien*(k-45)));
		yo3 = (int)(60 + 59 * cos(stopien*(k-45)));
	
		xo4 = (int)(62 + 59 * sin(stopien*(k-135)));
		yo4 = (int)(60 + 59 * cos(stopien*(k-135)));
		
		xw = (int)(62 + 59 * sin(stopien*(k+90)));
		yw = (int)(60 + 59 * cos(stopien*(k+90)));
	
		xs = (int)(62 + 59 * sin(stopien*(k+180)));
		ys = (int)(60 + 59 * cos(stopien*(k+180)));	
	
		xe = (int)(62 + 59 * sin(stopien*(k-90)));
		ye = (int)(60 + 59 * cos(stopien*(k-90)));
	
	  //Czyszczenie poprzednich znakow i rysowanie nowych
		TFTN_WriteXY(" \0" ,xns, yns,0x000,0x000, font2);
		TFTN_WriteXY("N\0" ,xn, yn,0xF00,0x000, font2);

		TFTN_WriteXY(" \0" ,xss, yss,0x000,0x000, font2);
		TFTN_WriteXY("S\0" ,xs, ys,0x00F,0x000, font2);
	
		TFTN_WriteXY(" \0" ,xws, yws,0x000,0x000, font2);
		TFTN_WriteXY("W\0" ,xw, yw,0xFFF,0x000, font2);
	
		TFTN_WriteXY(" \0" ,xes, yes,0x000,0x000, font2);
		TFTN_WriteXY("E\0" ,xe, ye,0xFFF,0x000, font2);
	
		TFTN_WriteXY(" \0" ,xo1s, yo1s,0x000,0x000, font1);
		TFTN_WriteXY("o\0" ,xo1, yo1,0xFFF,0x000, font1);
	
		TFTN_WriteXY(" \0" ,xo2s, yo2s,0x000,0x000, font1);
		TFTN_WriteXY("o\0" ,xo2, yo2,0xFFF,0x000, font1);
	
		TFTN_WriteXY(" \0" ,xo3s, yo3s,0x000,0x000, font1);
		TFTN_WriteXY("o\0" ,xo3, yo3,0xFFF,0x000, font1);	
		
		TFTN_WriteXY(" \0" ,xo4s, yo4s,0x000,0x000, font1);
		TFTN_WriteXY("o\0" ,xo4, yo4,0xFFF,0x000, font1);
		
		//Wyswietlenie kata na wyswietlaczu
		TFTN_WriteXY(Tekst ,52, 80,0x000,0x000, font1);
		TFTN_WriteXY(Tekst ,52, 80,0x0D0,0x000, font1);
			
		//Pobranie danych pola magnetycznego
		HMC5883L_GetPosition(&x, &y, &z);	
		
		//Obliczenie kierunku geograficznego
		//Skalowanie wartosci z czujnika i przesuniecie 
		//ukladu wspolrzednych
		scaledx=(x+offsetx)*m_Scale;
		scaledy=(y+offsety)*m_Scale;
		
		//Obliczenie kierunku geograficznego
		//heading = atan2(y, x);
		heading = atan2(scaledy, scaledx);
		heading += declinationAngle;

		if(heading < 0)
    heading += 6.2831853;   //2 pi
  
    if(heading > 6.2831853)
    heading -= 6.2831853;
		//Zamiana wyniku na stopnie
    headingDegrees = heading * 180/3.1415926; 
			
		//Przeksztalcenie liczby na stringa
		sprintf((char *)Tekst, "%4d\0", (signed short)headingDegrees);
		
		//W czasie pracy kompasu mruga 	LED 8
    GPIO_WriteBit(GPIOB, GPIO_Pin_15, (BitAction)(1-GPIO_ReadOutputDataBit(GPIOB, GPIO_Pin_15)));

		
	}
}	

void Opcja_Krzywe(void){
	float stopien = (2 * 3.1415926) / 360; 
	unsigned short kata=0,katb=0,katc=0;
	unsigned char a=0,b=0,c=0;
	volatile int xl1,yl1,xl2,yl2,xl3,yl3;
	volatile signed char seta,setb,setc;
	unsigned int kolor;	
	extern volatile unsigned char losowo;
	unsigned char Efekt[12] = {"\0"};	
	
	TFTN_Clear(0x000);
	seta=abs((rand()+losowo)%10+1);
	setb=abs((rand()+losowo)%10+1);
	setc=abs((rand()+losowo)%10+1);
	
	//Wyjscie do menu glownego przez JOY_DOWN
	while(GPIO_ReadInputDataBit(JOY_SW_PORT, JOY_DOWN))
	{
		a++;
		b++;
		c++;
		
		if (a==seta)
		{
		xl1 = (int)(66 + 20 * sin(stopien*kata));
		yl1 = (int)(66 + 20 * cos(stopien*kata));
		a=0;
			kata++;
			if(kata==359) kata=0;
		}
		
		if(b==setb)
		{
		xl2 = (int)(xl1 + 20 * sin(stopien*katb));
		yl2 = (int)(yl1 + 20 * cos(stopien*katb));
			b=0;
			katb++;
			if(katb==359) katb=0;
		}
	
		if(c==setc)
		{
		xl3 = (int)(xl2 + 20 * sin(stopien*katc));
		yl3 = (int)(yl2 + 20 * cos(stopien*katc));
			c=0;
			katc++;
			if(katc==359) katc=0;
		}
		
		//Zmiaana koloru w zalerznosci od figury
		kolor=((15-seta)<<8 | ((15-setb)*10)<<4 | ((15-setc)*100));
		
		//Ruysowanie figury
		TFTN_PutPixel(xl3, yl3, kolor);
		
		//Po wcisnieciu JOY_ENTER kolejny obrazek
		if (!GPIO_ReadInputDataBit(GPIOC, GPIO_Pin_5))
		{
			TFTN_Clear(0x000);
			
			//Losowa predkosc obrotu kazdej z 3 niewidocznych lini
			//
			//								Linia 1						 Linia 2						Linia 3
			//          0.0------------xl1,yl1------------xl2,yl2------------xl3,yl3(rysowanie piksela)
			//								 seta								setb							  setc
			//
			seta=abs((rand()+losowo)%10+1);
			setb=abs((rand()+losowo)%10+1);
			setc=abs((rand()+losowo)%10+1);
			a=0;
			b=0;
			c=0;
			
		//Predkosci przedstawione na LCD
		sprintf((char *)Efekt, "%2d %2d %2d\0", seta,setb,setc);
		TFTN_WriteXY(Efekt ,0, 3,0xFFF,0x000, font1);
		}
		
	}
}

						   
